using System;
using System.Collections.Generic;
using System.Text;
using System.Runtime.InteropServices;

namespace PCI3016_NET
{
    public class PCI3016
    {
        /// <summary>
        /// Store the index of the board
        /// </summary>
        private int index;

        /// <summary>
        /// Store an handle to the board. Will be used for interrupt routine for example
        /// </summary>
        private IntPtr handle;

        /// <summary>
        /// An object that will contain the timer event args
        /// Created only once, because the new command takes some time
        /// </summary>
        private CounterTimerWatchdogEventArgs CounterTimerWatchdogEventArgs;

        /// <summary>
        /// An object that will contain the analog input event args
        /// Created only once, because the new command takes some time
        /// </summary>
        private AnalogInputEventArgs analogInputEventArgs;

        /// <summary>
        /// An object that will contain the end of sequence event args
        /// Created only once, because the new command takes some time
        /// </summary>
        private EndOfSequenceEventArgs endOfSequenceEventArgs;

        /// <summary>
        /// Handler to the interrupt function of the board
        /// The function pointed by this delegate will be called each time the board interrupts
        /// </summary>
        private DllImport.InterruptFunctionEventHandler interruptFunctionHandler;

        #region "Event"

        /// <summary>
        /// The timer event
        /// </summary>
        public event CounterTimerWatchdogEventHandler CounterTimerWatchdogEventReceived;

        /// <summary>
        /// Delegate to provide when an interrupt on the timer occurs
        /// </summary>
        /// <param name="source"></param>
        /// <param name="args"></param>
        public delegate void CounterTimerWatchdogEventHandler(object source, CounterTimerWatchdogEventArgs args);

        /// <summary>
        /// The analog input event
        /// </summary>
        public event AnalogInputEventHandler AnalogInputEventReceived;

        /// <summary>
        /// Delegate to provide when an interrupt on the analog input occurs
        /// </summary>
        /// <param name="source"></param>
        /// <param name="args"></param>
        public delegate void AnalogInputEventHandler(object source, AnalogInputEventArgs args);

        /// <summary>
        /// The end of sequence event
        /// </summary>
        public event EndOfSequenceEventHandler EndOfSequenceEventReceived;

        /// <summary>
        /// Delegate to provide when an end of sequence interrupt occurs
        /// </summary>
        /// <param name="source"></param>
        /// <param name="args"></param>
        public delegate void EndOfSequenceEventHandler(object source, EndOfSequenceEventArgs args);

        /// <summary>
        /// The pld fifo overflow event
        /// </summary>
        public event PldFifoOverflowEventHandler PldFifoOverflowEventReceived;

        /// <summary>
        /// Delegate to provide when a PLD fifo overflow occurs
        /// </summary>
        /// <param name="source"></param>
        public delegate void PldFifoOverflowEventHandler(object source);

        #endregion

        #region "Constructor/Destructor"

        /// <summary>
        /// Constructor of the board
        /// Cannot be called; initialize the internal variables
        /// </summary>
        private PCI3016()
        {
            this.index = -1;
            this.handle = IntPtr.Zero;

            // Create the event args (once by board)
            this.CounterTimerWatchdogEventArgs = new CounterTimerWatchdogEventArgs();
            this.analogInputEventArgs = new AnalogInputEventArgs();
            this.endOfSequenceEventArgs = new EndOfSequenceEventArgs();

            // Create the interrupt handler (once by board)
            this.interruptFunctionHandler = new DllImport.InterruptFunctionEventHandler(InterruptFunction);

            // Tell the garbage collector to keep the pointer
            GC.KeepAlive(this.interruptFunctionHandler);
        }

        /// <summary>
        /// Destructor of the board
        /// Free all the ressources
        /// </summary>
        ~PCI3016()
        {
            DllImport.i_PCI3016_CloseBoard(handle);
            this.index = -1;
            this.handle = IntPtr.Zero;
        }

        #endregion

        #region "Utils"

        /// <summary>
        /// Enables to get an instance of a PCI3016 board
        /// </summary>
        /// <param name="index">The index of the board you want to open</param>
        /// <returns>null on error, instance of PCI3016 on success</returns>
        public static PCI3016 OpenBoard(int index)
        {
            PCI3016 boardInstance = null;
            int nbBoard = 0;
            IntPtr handle = IntPtr.Zero;

            // Get the number of boards
            if ((nbBoard = GetBoardNumber()) == -1)
            {
                Console.WriteLine("GetBoardNumber failed");
                return null;
            }

            // Check that the index is in the range
            if (index < 0 || index > nbBoard)
            {
                Console.WriteLine("index not valid");
                return null;
            }

            // Try to open the board
            if (DllImport.i_PCI3016_OpenBoardViaIndex((byte)index, ref handle) != 0)
            {
                Console.WriteLine("i_PCI3016_OpenBoardViaIndex failed");
                return null;
            }

            // Create a new instance of the board
            boardInstance = new PCI3016();

            // Set its parameters
            boardInstance.index = index;
            boardInstance.handle = handle;

            // Return the instance
            return boardInstance;
        }

        /// <summary>
        /// Enables to get the number of PCI3016 boards plugged in the computer
        /// </summary>
        /// <returns>The number of board</returns>
        public static int GetBoardNumber()
        {
            int ret = 0;
            byte nbBoard = 0;

            // Call the DLL function
            ret = DllImport.i_PCI3016_GetNumberOfBoards(ref nbBoard);

            // Error case
            if (ret != 0) { return -1; }

             // Success
            else { return (int)nbBoard; }
        }

        /// <summary>
        /// Enables to signal to all the listeners that a PLD fifo overflow occurs
        /// </summary>
        /// <param name="eventArgs">The arguments of the event</param>
        protected virtual void OnPldFifoOverflowEventReceived()
        {
            if (this.PldFifoOverflowEventReceived != null)
            {
                this.PldFifoOverflowEventReceived(this);
            }
        }

        #endregion

        #region "Interrupt"

        /// <summary>
        /// Interrupt function - Called each time the board generates an interrupt
        /// </summary>
        /// <param name="handle"></param>
        /// <param name="overflow"></param>
        /// <param name="source"></param>
        /// <param name="mask"></param>
        /// <param name="args"></param>
        /// <param name="argsSize"></param>
        private void InterruptFunction(IntPtr handle, byte overflow,uint source,uint mask,IntPtr args, uint argsSize )
        {
            // PLD fifo overflow ?
            if ((source & 0x10) == 0x10)
            {
                this.OnPldFifoOverflowEventReceived();
            }
            // Timer ?
            else if ( (source & 2) == 2)
            {
                this.CounterTimerWatchdogEventArgs.InterruptMask = mask;
                this.CounterTimerWatchdogEventArgs.FifoOverflow = overflow;

                // Signal the event
                this.OnCounterTimerWatchdogEventReceived(this.CounterTimerWatchdogEventArgs);
            }
            // DMA ?
            else if ((source & 4) == 4)
            {
                this.analogInputEventArgs.InterruptMask = mask;
                this.analogInputEventArgs.FifoOverflow = overflow;
                this.analogInputEventArgs.ArgsSize = argsSize;

                // Signal the event
                this.OnAnalogInputEventReceived(this.analogInputEventArgs);
            }
            // end of sequence ?
            else if ((source & 1) == 1)
            {
                byte[] data = new byte[argsSize * sizeof(ushort)];

                this.endOfSequenceEventArgs.InterruptMask = mask;
                this.endOfSequenceEventArgs.FifoOverflow = overflow;
                this.endOfSequenceEventArgs.ArgsSize = argsSize;

                // Convert the args
                Marshal.Copy(args, data, 0, (int)argsSize * sizeof(ushort));
                for (int i = 0; i < argsSize; i++)
                {
                    this.endOfSequenceEventArgs.Args[i] = BitConverter.ToUInt16(data, i * sizeof(ushort));
                }

                // Signal the event
                this.OnEndOfSequenceEventReceived(this.endOfSequenceEventArgs);
            }
        }

        /// <summary>
        /// Set the board interrupt routine
        /// </summary>
        /// <returns>0 on success, != 0 on error</returns>
        public int SetBoardIntRoutine()
        {
            return (int)DllImport.i_PCI3016_SetBoardIntRoutine(this.handle, interruptFunctionHandler);
        }

        /// <summary>
        /// Reset the board interrupt routine
        /// </summary>
        /// <returns></returns>
        public int ResetBoardIntRoutine()
        {
            return (int)DllImport.i_PCI3016_ResetBoardIntRoutine(this.handle);
        }

        #endregion

        #region "Digital outputs"

        /// <summary>
        /// Set the digital memory of the outputs to on
        /// </summary>
        /// <returns>0 on success, != 0 on error</returns>
        public int SetDigitalOutputMemoryOn()
        {
            return (int)DllImport.i_PCI3016_SetDigitalOutputMemoryOn(this.handle);
        }

        /// <summary>
        /// Set the digital memory of the outputs to off
        /// </summary>
        /// <returns></returns>
        public int SetDigitalOutputMemoryOff()
        {
            return (int)DllImport.i_PCI3016_SetDigitalOutputMemoryOff(this.handle);
        }

        /// <summary>
        /// Enables to set one channel to on (high state)
        /// </summary>
        /// <param name="channel">The index of the channel (start at 0)</param>
        /// <returns>0 on success, != 0 on error</returns>
        public int Set1DigitalOutputOn(byte channel)
        {
            return (int)DllImport.i_PCI3016_Set1DigitalOutputOn(this.handle, channel);
        }

        /// <summary>
        /// Enables to set one channel to off (low state)
        /// </summary>
        /// <param name="channel">The index of the channel (start at 0)</param>
        /// <returns>0 on success, != 0 on error</returns>
        public int Set1DigitalOutputOff(byte channel)
        {
            return (int)DllImport.i_PCI3016_Set1DigitalOutputOff(this.handle, channel);
        }

        /// <summary>
        /// Enables to set several digital outputs to on (high state)
        /// </summary>
        /// <param name="value">Digital output mask value to set (0 to 15)</param>
        /// <returns>0 on success, != 0 on error</returns>
        public int Set4DigitalOutputsOn(byte value)
        {
            return (int)DllImport.i_PCI3016_Set4DigitalOutputsOn(this.handle, value);
        }

        /// <summary>
        /// Enables to set several digital outputs to off (low state)
        /// </summary>
        /// <param name="value">Digital output mask value to set (0 to 15)</param>
        /// <returns>0 on success, != 0 on error</returns>
        public int Set4DigitalOutputsOff(byte value)
        {
            return (int)DllImport.i_PCI3016_Set4DigitalOutputsOff(this.handle, value);
        }

        /// Read 1 digital output status
        /// </summary>
        /// <param name="channel">Channel to be read (0 to 31)</param>
        /// <param name="outputStatus">Status of the digital outputs</param>
        /// <returns>0 on success, != 0 on error</returns>
        public int Get1DigitalOutputsStatus(byte channel, ref byte outputStatus)
        {
            return (int) DllImport.i_PCI3016_Get1DigitalOutputStatus(this.handle, channel, ref outputStatus);
        }

        /// <summary>
        /// Read 16 digital outputs status
        /// </summary>
        /// <param name="value">Status of the digital outputs</param>
        /// <returns>0 on success, != 0 on error</returns>
        public int Get4DigitalOutputsStatus(ref byte outputStatus)
        {
            return (int)DllImport.i_PCI3016_Get4DigitalOutputsStatus(this.handle, ref outputStatus);
        }

        #endregion

        #region "Digital inputs"

        /// <summary>
        /// Enables to read one digital
        /// </summary>
        /// <param name="channel">Channel to read (0-8)</param>
        /// <param name="inputValue">A byte that will be filled with the current status of the input (0,1)</param>
        /// <returns>0 on success, != 0 on error</returns>
        public int Read1DigitalInput(byte channel, ref byte inputValue)
        {
            // Call the function
            return DllImport.i_PCI3016_Read1DigitalInput(this.handle, channel, ref inputValue);
        }

        /// <summary>
        /// Enables to read 4 digital inputs (per port)
        /// </summary>
        /// <param name="inputValues">A byte that will be filled with the current status of the inputs</param>
        /// <returns>0 on success, != 0 on error</returns>
        public int Read4DigitalInputs(ref byte inputValues)
        {
            // Call the function
            return DllImport.i_PCI3016_Read4DigitalInputs(this.handle, ref inputValues);
        }

        #endregion

        #region "Counter - Timer - Watchdog"

        /// <summary>
        /// Enables to signal to all the listeners that an event on the timer occurs
        /// </summary>
        /// <param name="eventArgs">The arguments of the event</param>
        protected virtual void OnCounterTimerWatchdogEventReceived(CounterTimerWatchdogEventArgs eventArgs)
        {
            if (this.CounterTimerWatchdogEventReceived != null)
            {
                this.CounterTimerWatchdogEventReceived(this, eventArgs);
            }
        }

        /// <summary>
        /// Initialize and start a component as a counter, timer or watchdog
        /// </summary>
        /// <param name="function">Function choosed. 0: counter, 1: timer, 2: watchdog</param>
        /// <param name="index">Index of the component (0 to 2)</param>
        /// <param name="interruptFlag">0: without interrupt, 1: with interrupt</param>
        /// <param name="reloadValue">For the counter: its maximum value. For timer and watchdog: the time to count (depend on timerWatchdogTimeBase)</param>
        /// <param name="timerWatchdogTimeBase">The time base. 0: microseconds, 1: millseconds, 2: seconds</param>
        /// <param name="counterTimerOutputMode">The output mode. 0: disabled, 1: enabled, 2: inverted</param>
        /// <param name="timerMode">The timer mode (see documentation). 0: mode 2, 1: mode 3</param>
        /// <param name="counterDirection">The counter direction. 0: up, 1: down</param>
        /// <param name="counterLevel">Counter counts low levels, or high levels or both. 0: low, 1: high, 2: both</param>
        /// <returns>0 on success, != 0 on error</returns>
        public int InitAndStartCounterTimerWatchdog(byte function,
            byte index,
            byte interruptFlag,
            ushort reloadValue,
            byte timerWatchdogTimeBase,
            byte counterTimerOutputMode,
            byte timerMode,
            byte counterDirection,
            byte counterLevel)
        {
            return DllImport.i_PCI3016_InitAndStartCounterTimerWatchdog(this.handle,
                function,
                index,
                interruptFlag,
                reloadValue,
                timerWatchdogTimeBase,
                counterTimerOutputMode,
                timerMode,
                counterDirection,
                counterLevel);
        }

        /// <summary>
        /// Stop a counter - timer - watchdog
        /// </summary>
        /// <param name="index">The index of the component to stop</param>
        /// <returns>0 on success, != 0 on error</returns>
        public int StopCounterTimerWatchdog(byte index)
        {
            return DllImport.i_PCI3016_StopCounterTimerWatchdog(this.handle, index);
        }

        /// <summary>
        /// Trigger a counter - timer - watchdog
        /// </summary>
        /// <param name="index">The index of the component to trigger</param>
        /// <returns>0 on success, != 0 on error</returns>
        public int TriggerCounterTimerWatchdog(byte index)
        {
            return DllImport.i_PCI3016_TriggerCounterTimerWatchdog(this.handle, index);
        }

        /// <summary>
        /// Read the status of the counter
        /// </summary>
        /// <param name="index">Index of the component</param>
        /// <param name="status">0: disabled, 1: enabled</param>
        /// <param name="clear">0: counter has not been cleard, 1: counter has been cleared</param>
        /// <param name="softwareTriggerStatus">0: no software trigger occured, 1: software trigger occured</param>
        /// <param name="value">Current value of the counter</param>
        /// <returns>0 on success, != 0 on error</returns>
        public int ReadCounterStatus(byte index,
            ref byte status,
            ref byte clear,
            ref byte softwareTriggerStatus,
            ref ushort value)
        {
            return DllImport.i_PCI3016_ReadCounterStatus(this.handle,
                index,
                ref status,
                ref clear,
                ref softwareTriggerStatus,
                ref value);
        }

        /// <summary>
        /// Read the value (and only the value) of the counter
        /// </summary>
        /// <param name="index">Index of the component</param>
        /// <param name="value">Current value of the counter</param>
        /// <returns>0 on success, != 0 on error</returns>
        public int ReadCounterValue(byte index,
            ref ushort value)
        {
            return DllImport.i_PCI3016_ReadCounterValue(this.handle,
                index,
                ref value);
        }

        /// <summary>
        /// Clear the counter
        /// </summary>
        /// <param name="index">Index of the component</param>
        /// <returns>0 on success, != 0 on error</returns>
        public int ClearCounter(byte index)
        {
            return DllImport.i_PCI3016_ClearCounter(this.handle, index);
        }

        /// <summary>
        /// Read the status of the timer
        /// </summary>
        /// <param name="index">Index of the component</param>
        /// <param name="status">0: disabled, 1: enabled</param>
        /// <param name="softwareTriggerStatus">0: no software trigger occured, 1: software trigger occured</param>
        /// <param name="value">Current value of the timer</param>
        /// <returns>0 on success, != 0 on error</returns>
        public int ReadTimerStatus(byte index,
            ref byte status,
            ref byte softwareTriggerStatus,
            ref ushort value)
        {
            return DllImport.i_PCI3016_ReadTimerStatus(this.handle,
                index,
                ref status,
                ref softwareTriggerStatus,
                ref value);
        }

        /// <summary>
        /// Read the value of the timer
        /// </summary>
        /// <param name="index">Index of the component</param>
        /// <param name="value">Current value of the timer</param>
        /// <returns>0 on success, != 0 on error</returns>
        public int ReadTimerValue(byte index,
            ref ushort value)
        {
            return DllImport.i_PCI3016_ReadTimerValue(this.handle,
                index,
                ref value);
        }

        /// <summary>
        /// Read the current status of the watchdog
        /// </summary>
        /// <param name="status">0: disabled, 1: enabled</param>
        /// <param name="softwareTriggerStatus">0: no software trigger occured, 1: software trigger occured</param>
        /// <param name="value">Current value of the watchdog</param>
        /// <returns>0 on success, != 0 on error</returns>
        public int ReadWatchdogStatus(ref byte status,
            ref byte softwareTriggerStatus,
            ref ushort value)
        {
            return DllImport.i_PCI3016_ReadWatchdogStatus(this.handle,
                ref status,
                ref softwareTriggerStatus,
                ref value);
        }

        /// <summary>
        /// Read the current value of the watchdog
        /// </summary>
        /// <param name="value">Current value of the watchdog</param>
        /// <returns>0 on success, != 0 on error</returns>
        public int ReadWatchdogValue(ref ushort value)
        {
            return DllImport.i_PCI3016_ReadWatchdogValue(this.handle,
                ref value);
        }

        #endregion

        #region "Analog inputs"

        /// <summary>
        /// Enables to signal to all the listeners that an event on the analog inputs occurs
        /// </summary>
        /// <param name="eventArgs">The arguments of the event</param>
        protected virtual void OnAnalogInputEventReceived(AnalogInputEventArgs eventArgs)
        {
            if (this.AnalogInputEventReceived != null)
            {
                this.AnalogInputEventReceived(this, eventArgs);
            }
        }

        /// <summary>
        /// Enables to signal to all the listeners that an end of sequence interrupt occurs
        /// </summary>
        /// <param name="eventArgs"></param>
        protected virtual void OnEndOfSequenceEventReceived(EndOfSequenceEventArgs eventArgs)
        {
            if (this.EndOfSequenceEventReceived != null)
            {
                this.EndOfSequenceEventReceived(this, eventArgs);
            }
        }

        /// <summary>
        /// Set the input mode of the board
        /// </summary>
        /// <param name="inputMode">1: single ended, 2: differential</param>
        /// <returns>0 on success, != 0 on error</returns>
        public int SetAnalogInputMode(byte inputMode)
        {
            return DllImport.i_PCI3016_SetAnalogInputMode(this.handle, inputMode);
        }

        /// <summary>
        /// Get the current input mode of the board
        /// </summary>
        /// <param name="inputMode">Will be filled with the current input mode. 1: single ended, 2: differential</param>
        /// <returns>0 on success, != 0 on error</returns>
        public int GetAnalogInputMode(ref byte inputMode)
        {
            return DllImport.i_PCI3016_GetAnalogInputMode(this.handle, ref inputMode);
        }

        /// <summary>
        /// Initialize and start a sequence acquisition
        /// </summary>
        /// <param name="sequenceCount">The number of sequence to acquire. Set to 0 if continuous acquisition</param>
        /// <param name="sequenceBeforeInterrupt">The number of sequence to be acquired per interrupt</param>
        /// <param name="channelCount">The number of channels in one sequence</param>
        /// <param name="channelArray">An array that contains the index of the channels that will be acquired</param>
        /// <param name="gainArray">An array that contains the gain of the channels</param>
        /// <param name="polarityArray">An array that contains the polarity of the channels</param>
        /// <param name="convertingTime">The converting time 5us to 65 535 000us</param>
        /// <param name="delayTime">Define the time between two sequences acquisition (5us to 65 535 000us)</param>
        /// <param name="softTriggerMode">Define the software trigger</param>
        /// <param name="extTriggerMode">Define the hardware trigger</param>
        /// <param name="sequenceTriggerCount">If trigger is enabled, and sequence mode is choosed, define the number of sequence that is acquired per trigger</param>
        /// <returns>0 on success, != 0 on error</returns>
        public int InitAndStartAnalogInputSequence(uint sequenceCount,
            uint sequenceBeforeInterrupt,
            byte channelCount,
            byte[] channelArray,
            byte[] gainArray,
            byte[] polarityArray,
            uint convertingTime,
            uint delayTime,
            byte softTriggerMode,
            byte extTriggerMode,
            uint sequenceTriggerCount)
        {
            return DllImport.i_PCI3016_InitAndStartAnalogInputSequence(this.handle, sequenceCount, sequenceBeforeInterrupt, channelCount,
                channelArray, gainArray, polarityArray, convertingTime, delayTime, softTriggerMode, extTriggerMode, sequenceTriggerCount);
        }

        /// <summary>
        /// Stop any analog input acquisition that is running
        /// </summary>
        /// <returns>0 on success, != 0 on error</returns>
        public int StopAnalogAcquisition()
        {
            return DllImport.i_PCI3016_StopAnalogAcquisition(this.handle);
        }

        /// <summary>
        /// Enables to read the data from an analog input sequence
        /// </summary>
        /// <param name="size">Size of data to read</param>
        /// <param name="data">Pointer that will be filled with the read data</param>
        /// <returns> >= 0: number of data read, -1: error </returns>
        public int ReadSequenceData(uint size, ushort[] data)
        {
            return DllImport.l_PCI3016_ReadSequenceData(this.handle, size, data);
        }

        /// <summary>
        /// Enable to read more analog inputs
        /// </summary>
        /// <param name="channelCount">The number of channels to read</param>
        /// <param name="channelArray">An array that contains the index of the channels that will be acquired</param>
        /// <param name="gainArray">An array that contains the gain of the channels</param>
        /// <param name="polarityArray">An array that contains the polarity of the channels</param>
        /// <param name="convertingTime">The converting time 5us to 65 535 000us</param>
        /// <param name="interruptFlag">0: no interrupt, 1: interrupt</param>
        /// <param name="softTriggerMode">Define the software trigger</param>
        /// <param name="extTriggerMode">Define the hardware trigger</param>
        /// <param name="value">An array that will be filled the values of the channels</param>
        /// <returns>0 on success, != 0 on error</returns>
        public int ReadMoreAnalogInputs(byte channelCount,
            byte[] channelArray,
            byte[] gainArray,
            byte[] polarityArray,
            uint convertingTime,
            byte interruptFlag,
            byte softTriggerMode,
            byte extTriggerMode,
            ushort[] value)
        {
            return DllImport.i_PCI3016_ReadMoreAnalogInputs(this.handle, channelCount, channelArray, gainArray, polarityArray, convertingTime, interruptFlag, softTriggerMode, extTriggerMode, value);
        }

        /// <summary>
        /// Read one analog input of the board
        /// </summary>
        /// <param name="channelIndex">The channel index of the input (0 - 15)</param>
        /// <param name="gain">The gain of the channel</param>
        /// <param name="polarity">The polarity of the channel</param>
        /// <param name="convertingTime">The converting time 5us to 65 535 000us</param>
        /// <param name="interruptFlag">0: no interrupt, 1: interrupt</param>
        /// <param name="softTriggerMode">Define the software trigger</param>
        /// <param name="extTriggerMode">Define the hardware trigger</param>
        /// <param name="value">A reference to the value that will be read</param>
        /// <returns>0 on success, != 0 on error</returns>
        public int Read1AnalogInput(byte channelIndex,
            byte gain,
            byte polarity,
            uint convertingTime,
            byte interruptFlag,
            byte softTriggerMode,
            byte extTriggerMode,
            ref ushort value)
        {
            return DllImport.i_PCI3016_Read1AnalogInput(this.handle, channelIndex, gain, polarity, convertingTime, interruptFlag, softTriggerMode, extTriggerMode, ref value);
        }

        /// <summary>
        /// Initialize and start an autorefresh acquisition
        /// </summary>
        /// <param name="channelCount">The number of channels to read</param>
        /// <param name="channelArray">An array that contains the index of the channels that will be acquired</param>
        /// <param name="gainArray">An array that contains the gain of the channels</param>
        /// <param name="polarityArray">n array that contains the polarity of the channels</param>
        /// <param name="convertingTime">The converting time 5us to 65 535 000us</param>
        /// <param name="value">A reference to an array that will be updated with the value of the channels</param>
        /// <returns>0 on success, != 0 on error</returns>
        public int InitAndStartAutoRefreshAcquisition(byte channelCount,
            byte[] channelArray,
            byte[] gainArray,
            byte[] polarityArray,
            uint convertingTime,
            ref IntPtr value)
        {
            return DllImport.i_PCI3016_InitAndStartAutoRefreshAcquisition(this.handle, channelCount, channelArray, gainArray, polarityArray, convertingTime, ref value);
        }

        /// <summary>
        /// Trigger a current analog input acquisition (software)
        /// </summary>
        /// <returns>0 on success, != 0 on error</returns>
        public int TriggerAnalogAcquisition()
        {
            return DllImport.i_PCI3016_TriggerAnalogAcquisition(this.handle);
        }

        /// <summary>
        /// Get the number of analog inputs of the board
        /// </summary>
        /// <param name="analogInputCount">The number of analog inputs</param>
        /// <returns>0 on success, != 0 on error</returns>
        public int GetAnalogInputChannelsNbr(ref byte analogInputCount)
        {
            return DllImport.i_PCI3016_GetAnalogInputChannelsNbr(this.handle, ref analogInputCount);
        }

        /// <summary>
        /// Get the resolution of the analog inputs of the board
        /// </summary>
        /// <param name="resolution">The resolution of the analog inputs of the board, in bits</param>
        /// <returns>0 on success, != 0 on error</returns>
        public int GetAnalogInputResolution(ref byte resolution)
        {
            return DllImport.i_PCI3016_GetAnalogInputResolution(this.handle, ref resolution);
        }

        #endregion

        #region "TTL inputs"

        public int ReadTTLInputs(byte port, ref UInt16 inputValue)
        {
            // Call the function
            return DllImport.i_PCI3016_ReadTTLInputs(this.handle, port, ref inputValue);
        }

        public int SetTTLPortConfiguration()
        {
            // Call the function
            byte port = 1;
            return DllImport.i_PCI3016_SetTTLPortConfiguration(this.handle, port);
        }

        #endregion

        #region "TTL outputs"

        public int SetTTLOutputsOff(UInt16 mask)
        {
            // Call the function
            byte port = 1;
            return DllImport.i_PCI3016_SetTTLOutputsOff(this.handle, port, mask);
        }

        public int GetTTLOutputsStatus(ref UInt16 outputStatus)
        {
            // Call the function
            byte port = 1;
            return DllImport.i_PCI3016_GetTTLOutputsStatus(this.handle, port, ref outputStatus);
        }

        public int SetTTLOutputsOn(UInt16 mask)
        {
            // Call the function
            byte port = 1;
            return DllImport.i_PCI3016_SetTTLOutputsOn(this.handle, port, mask);
        }


        #endregion
    }
}
